unit ECDSADemoMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, stCustomProtoComp, stSecComp, StreamSec.DSI.ArithComp,
  ExtCtrls;

type
  TfrmMain = class(TForm)
    btnCreateKey: TButton;
    memPrivKey: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    memPublKey: TMemo;
    rgKeySize: TRadioGroup;
    btnSign: TButton;
    Label3: TLabel;
    btnVerify: TButton;
    memMsg: TMemo;
    Signature: TLabel;
    edtSign: TEdit;
    lblCreationTime: TLabel;
    lblSignStat: TLabel;
    rgDigestAlg: TRadioGroup;
    ECDSAKey: TstECDSAKey;
    btnLoad: TButton;
    procedure btnLoadClick(Sender: TObject);
    procedure memPrivKeyChange(Sender: TObject);
    procedure rgKeySizeClick(Sender: TObject);
    procedure btnVerifyClick(Sender: TObject);
    procedure btnSignClick(Sender: TObject);
    procedure btnCreateKeyClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  StreamSec.DSI.ECC, stARC4, stSHA1, stRijndaelBG, stGlobalYarrow,
  StreamSec.DSI.CAPIFormat, StreamSec.DSI.NETFormat, StreamSec.DSI.PKTypes,
  stSecUtils, stSHA256, stSHA512, StreamSec.DSI.ECC.NISTCurves,
  StreamSec.DSI.ECC.X9_62Curves, StreamSec.DSI.ECArith;

{$R *.dfm}

procedure TfrmMain.btnCreateKeyClick(Sender: TObject);
var
  lTicks: Cardinal;
  lSS: TStringStream;
begin
  ECDSAKey.Enabled := False;            
  case rgDigestAlg.ItemIndex of
    0: ECDSAKey.DigestAlgorithm := haSHA1;
    1: ECDSAKey.DigestAlgorithm := haSHA224;
    2: ECDSAKey.DigestAlgorithm := haSHA256;
    3: ECDSAKey.DigestAlgorithm := haSHA384;
    4: ECDSAKey.DigestAlgorithm := haSHA512;
  end;
  lTicks := GetTickCount;
  case rgKeySize.ItemIndex of
    0: ECDSAKey.GenerateNewKeys(prime192v1,False);
    1: ECDSAKey.GenerateNewKeys(prime192v2,False);
    2: ECDSAKey.GenerateNewKeys(prime192v3,False);
    3: ECDSAKey.GenerateNewKeys(prime239v1,False);
    4: ECDSAKey.GenerateNewKeys(prime239v2,False);
    5: ECDSAKey.GenerateNewKeys(prime239v3,False);
    6: ECDSAKey.GenerateNewKeys(prime256v1,False);
    7: ECDSAKey.GenerateNewKeys(prime224,False);
    8: ECDSAKey.GenerateNewKeys(prime384,False);
    9: ECDSAKey.GenerateNewKeys(prime521,False);
  end;
  lTicks := GetTickCount - lTicks;
  ECDSAKey.Enabled := True;
  lblCreationTime.Caption := Format('Key pair created in %d ms',[lTicks]);
  lSS := TStringStream.Create('');
  try
    ECDSAKey.SavePrivateKeyToStream(lSS,iECDSAXmlFormat);
    memPrivKey.Text := StringReplace(lSS.DataString,'><','>'#13#10'<',[rfReplaceAll]);
    lSS.Size := 0;
    ECDSAKey.SavePublicKeyToStream(lSS,iECDSAXmlFormat);
    memPublKey.Text := StringReplace(lSS.DataString,'><','>'#13#10'<',[rfReplaceAll]);
  finally
    lSS.Free;
  end;
  btnSign.Enabled := True;
end;

procedure TfrmMain.btnSignClick(Sender: TObject);
begin
  edtSign.Text := ECDSAKey.GenerateSignature(memMsg.Text,sfBase64);
  btnVerify.Enabled := True;
end;

procedure TfrmMain.btnVerifyClick(Sender: TObject);
var
  lTicks: Cardinal;
  lOK: Boolean;
begin
  lTicks := GetTickCount;
  lOK := ECDSAKey.VerifySignature(memMsg.Text,edtSign.Text,sfBase64);
  lTicks := GetTickCount - lTicks;
  if lOK then
    lblSignStat.Caption := Format('OK - Verified in %d ms',[lTicks])
  else
    lblSignStat.Caption := 'Failure';
end;

procedure TfrmMain.rgKeySizeClick(Sender: TObject);
begin
  btnSign.Enabled := False;
  btnVerify.Enabled := False;
end;

procedure TfrmMain.memPrivKeyChange(Sender: TObject);
begin
  btnLoad.Enabled := (memPrivKey.Text <> '') and (memPublKey.Text <> '');
end;

procedure TfrmMain.btnLoadClick(Sender: TObject);
var
  lSS: TStringStream;
  lPwd: iSecretKey;
  lPriv: iDLPrivateKey;
  lPubl: iDLPublicKey;
begin
  ECDSAKey.Enabled := False;
  lSS := TStringStream.Create(memPrivKey.Text);
  try
    ECDSAKey.SetPrivateKey(ECDSAKey.PrivateKeyFormat.LoadFromStream(lPwd,lSS,lPriv));
  finally
    lSS.Free;
  end;                    
  lSS := TStringStream.Create(memPublKey.Text);
  try
    ECDSAKey.SetPublicKey(ECDSAKey.PublicKeyFormat.LoadFromStream(lSS,lPubl));
  finally
    lSS.Free;
  end;
  ECDSAKey.Enabled := True;
end;

end.
